@@ -337,6 +337,8 @@ sub build_btrfs_commands {
     }
   }
 
+  my %mkfs_done;
+
   foreach my $config (keys %FAI::configs) { # loop through all configs
     next unless ($config eq "BTRFS");
 
@@ -366,15 +368,22 @@ sub build_btrfs_commands {
     }
     # creates the BTRFS volume/RAID
     if ($raidlevel eq 'single') {
-          &FAI::push_command("mkfs.btrfs -d single $createopts ". join(" ",@devs),
-                             "$pre_req",
-                             "btrfs_built_raid_$id");
-
-        } else {
-          &FAI::push_command("mkfs.btrfs -d raid$raidlevel $createopts ". join(" ",@devs),
-                             "$pre_req",
-                             "btrfs_built_raid_$id");
-        }
+      if (exists $mkfs_done{join(" ", @devs)}) {
+        &FAI::push_command("true",
+			   "$pre_req",
+			   "btrfs_built_raid_$id");
+      } else {
+        print "Adding mkfs command for '", join(", ", @devs), "'.\n";
+        &FAI::push_command("mkfs.btrfs -d single $createopts ".join(" ",@devs),
+			   "$pre_req",
+			   "btrfs_built_raid_$id");
+	$mkfs_done{join(" ", @devs)} = '1';
+      }
+    } else {
+      &FAI::push_command("mkfs.btrfs -d raid$raidlevel $createopts ".join(" ",@devs),
+			 "$pre_req",
+			 "btrfs_built_raid_$id");
+    }
 
     # initial mount, required to create the initial subvolume
     &FAI::push_command("mount $devs[0] /mnt",
@@ -1299,6 +1308,10 @@ sub setup_partitions {
     if (defined $part->{mountpoint} && $part->{mountpoint} eq "/boot") {
       $boot_disk=$disk;
     }
+    # override $boot_disk if /boot/efi exists
+    if (defined $part->{mountpoint} && $part->{mountpoint} eq "/boot/efi") {
+      $boot_disk=$disk;
+    }
 
     # the type of the partition defaults to primary
     my $part_type = "primary";
@@ -1349,9 +1362,10 @@ sub setup_partitions {
     $prev_id = $part_id;
   }
 
+  # set bootable flag for gpt-bios and gpt
   &FAI::push_command("parted $boot_disk set 1 boot on",
     "pt_complete_$disk", "gpt_bios_fake_bootable")
-    if($FAI::configs{$config}{disklabel} eq "gpt-bios" and $boot_disk);
+    if($FAI::configs{$config}{disklabel} =~ /^gpt/ and $boot_disk);
 
   ($prev_id > -1) or &FAI::internal_error("No partitions created");
   $partition_table_deps{$disk} = "cleared2_$disk,exist_"
